/*
 * QuickTime for Java SDK Sample Code

   Usage subject to restrictions in SDK License Agreement
 * Copyright:  1996-1999 Apple Computer, Inc.

 */
import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.io.File;

import quicktime.*;
import quicktime.qd.*;
import quicktime.std.*;
import quicktime.io.*;
import quicktime.std.image.*;
import quicktime.util.*;
import quicktime.std.movies.*;
import quicktime.std.movies.media.*;
import quicktime.app.actions.*;
import quicktime.app.anim.*;
import quicktime.app.audio.*;
import quicktime.app.display.*;
import quicktime.app.image.*;
import quicktime.app.QTFactory;

import ds.actions.*;

import quicktime.app.event.*;
public final class DraggingSprites extends Frame implements StdQTConstants {
	static final int 
				kWidth = 400,
				kHeight = 300;
	
	// MIDI note numbers for drum kit instrument
	static final int
		kSnareDrum = 40,
		kClosedHiHat = 44,
		kOpenHiHat = 46,
		kCrashCymbal = 49,
		kRideCymbal = 51;

	public static void main (String[] args) {
		try {
			QTSession.open();
			new DraggingSprites().show();
		} catch (Exception e) {
			e.printStackTrace();
			QTSession.close();
		}
	}
	
//____________________ INSTANCE VARIABLES	
	private Compositor myCompositor;
	private QTCanvas canvas;
	
//____________________ INSTANCE METHODS	
	DraggingSprites () throws QTException, IOException {
		super ("Sprite App");

		canvas = new QTCanvas (QTCanvas.kInitialSize, 0.5F, 0.5F);
		add ("Center", canvas);
		QDDimension size = new QDDimension(kWidth, kHeight);
		QDGraphics gw = new QDGraphics (QDConstants.k16BE555PixelFormat, new QDRect(size));
		
		QDGraphics backBuffer = new QDGraphics (QDConstants.k16BE555PixelFormat, new QDRect (size));
		
		myCompositor = new Compositor (gw, QDColor.red, backBuffer, 20, 1);	
		
		File matchFile = QTFactory.findAbsolutePath ("Ships.mov");
		OpenMovieFile movieFile = OpenMovieFile.asRead(new QTFile(matchFile));
		Movie mov = Movie.fromFile (movieFile);
		Track shipTrack = mov.getIndTrack (1);
		ImageDataSequence ids = ImageUtil.createSequence (shipTrack);
		
		createSprites (myCompositor, ImageUtil.makeTransparent (ids, QDColor.blue), size);
		myCompositor.getTimer().setRate (2);
		canvas.setClient (myCompositor, true);

		addWindowListener(new WindowAdapter () {
			public void windowClosing (WindowEvent e) {
				QTSession.close();
				dispose();
			}

			public void windowClosed (WindowEvent e) {
				System.exit(0);
			}
		});
		pack();
	}
				
	void createSprites (Compositor myCompositor, ImageDataSequence seq, QDDimension size) throws QTException, IOException {
// SET UP SPRITES
	// two spaceships
		Matrix matrix = new Matrix();
		matrix.setTx(size.getWidth() / 4);
		matrix.setTy(size.getHeight() / 2);
		TwoDSprite firstSpaceShip = new TwoDSprite(seq, matrix, true, 1);
		myCompositor.addMember (firstSpaceShip);
		
 		matrix = new Matrix();
		matrix.setTx(size.getWidth() / 2);
		matrix.setTy(size.getHeight() / 2);
		TwoDSprite rotatedSprite = new TwoDSprite(seq, 15, matrix, true, 10);
		myCompositor.addMember (rotatedSprite);
		ImageSequencer is3 = new ImageSequencer (seq);
		is3.setCurrentFrame (15);	//keep in sync with sprite's current image
		is3.setLooping (ImageSequencer.kLoopForwards);

			//set a background image sprite into the back layer
			// this insures better drawing with a static image.
		QTFile backgroundFile = new QTFile (QTFactory.findAbsolutePath("pics/stars.jpg"));
		GraphicsImporterDrawer importerDrawer1 = new GraphicsImporterDrawer (backgroundFile);
		importerDrawer1.setDisplayBounds (new QDRect (kWidth, kHeight));
		myCompositor.addMember (importerDrawer1, Layerable.kBackMostLayer);

	// paint a sprite and set a hot spot region for it
	// the blue region is the hot spot
	// we use a 16 bit BigEndian pixel format as we have no blending on this sprite
		matrix = new Matrix();
		matrix.setTx(0);
		matrix.setTy(0);
		QDRect r40 = new QDRect (40, 40);
		QDRect r20 = new QDRect (20, 20);
		QDGraphics offscreen2 = new QDGraphics (QDConstants.k16BE555PixelFormat, r40);
		offscreen2.setBackColor (QDColor.yellow);
		offscreen2.eraseRect (null);
		offscreen2.setForeColor (QDColor.blue);
		offscreen2.paintRect (r20);
		EncodedImage ei = RawEncodedImage.fromPixMap (offscreen2.getPixMap());
		ImageDescription id = new ImageDescription (offscreen2.getPixMap());
		ImageDataSequence idsy = new ImageDataSequence (id);
		idsy.addMember (ei);
		idsy = ImageUtil.makeTransparent (idsy, QDColor.black, new QDGraphics (QDConstants.k16BE555PixelFormat, r40), new Region (r20));
		TwoDSprite colouredSquareSprite = new TwoDSprite(idsy, matrix, true, 10);
		myCompositor.addMember (colouredSquareSprite);


		// This is the instrument that will be used to play all of the notes
		// of the different actions -> we use a polyphony of 10
		// which is a reasonable estimate of the maximum number of notes
		// that will be sounding at a given time
		final NoteChannelControl notePlayer = new NoteChannelControl (StdQTConstants.kFirstDrumkit, 10);

// SET UP a set of timed actions to
	//1. flip the images for the spaceships (NextImageAction)
	//2. move the spaceships around
	//3. rotate one of the spaceships
		SimpleActionList actionList = new SimpleActionList();
		ImageSequencer imgSequencer = new ImageSequencer (seq);
		imgSequencer.setLooping (ImageSequencer.kLoopForwards);
		actionList.addMember (new NextImageAction (15, 1, imgSequencer, firstSpaceShip));
		BounceAction bounce1 = new BounceAction (20, 1, myCompositor, firstSpaceShip, 1, 2);
		bounce1.setActionable (new Actionable () {
			public void trigger () {
				try {
					notePlayer.playNoteFor (kCrashCymbal, 127, 20);
				} catch (QTException e) {
					e.printStackTrace();
				}
			}
		});
		actionList.addMember (bounce1);
		actionList.addMember (new MatrixAction (20, 1, myCompositor, rotatedSprite, new RotateMatrix ()));
					// 360/ (1/2)number of images changes at a scale of 20
		actionList.addMember (new NextImageAction ((360 / seq.size() / 2), 20, is3, rotatedSprite));
		actionList.addMember (new BounceAction (1, 10, myCompositor, rotatedSprite, 100, 1)); // jump
		myCompositor.addController(actionList);

// SET UP some user mouse control actions
	// ONE controller with several behaviours that are applied to ALL sprites -> wholespace == true
		QTMouseTargetController allSpritesController = new QTMouseTargetController (true);
		myCompositor.addController (allSpritesController);
	// 1st Behaviour -> drag all sprites around when NO modifiers are pressed
	// - play notes while dragging.
		allSpritesController.addQTMouseListener (new DragAction (new TranslateMatrix()) {
				// we do this to set the actionable of the DragAction to play a note
				// when it bounces
			{
			 	setActionable (new Actionable () {
					public void trigger () {
						try {
							notePlayer.playNoteFor (kRideCymbal, 127, 80);
						} catch (QTException e) {
							e.printStackTrace();
						}
					}
				});			
			}

			//only activates when NO modifiers are pressed
			public boolean matchModifierFilter (int mods) {
				if ((mods & 0xF) == 0)
					return true;
				return false;
			}
		    public void mouseDragged(QTMouseEvent event) {
		    	super.mouseDragged (event);
				try {
					notePlayer.playNoteFor (kClosedHiHat, 127, 80);
				} catch (QTException e) {
					e.printStackTrace();
				}
		    }
		});
	// 2nd Behaviour -> apply a blend mode when the mouse enters any sprite
		allSpritesController.addQTMouseListener (new MouseTargetAdapter () {
			GraphicsMode savedGM;
			GraphicsMode setGM = new GraphicsMode (QDConstants.blend, QDColor.lightGray);

			public void mouseTargetEntered (QTMouseEvent event) {
				try {
					TwoDSprite sprite = (TwoDSprite)event.getTarget();
					savedGM = sprite.getGraphicsMode();
					sprite.setGraphicsMode (setGM);
					notePlayer.playNoteFor (kSnareDrum, 127, 80);
				} catch (QTException e) {
					throw new QTRuntimeException (e);
				}
			}

			public void mouseTargetExited (QTMouseEvent event) {
				try {
					TwoDSprite sprite = (TwoDSprite)event.getTarget();
					sprite.setGraphicsMode (savedGM);
					notePlayer.playNoteFor (kOpenHiHat, 127, 80);
				} catch (QTException e) {
					throw new QTRuntimeException (e);
				}
			}
		});

	// SECOND controller with several behaviours that are applied to only two of the sprites -> wholespace == false
	// and we explicitly add the sprites we want this controller to deal with
	//1st BEHAVIOUR -> only shift key down when mouse pressed will allow specified sprites to be scaled when dragged
		QTMouseTargetController specifiedSpritesController = new QTMouseTargetController (false);
		myCompositor.addController (specifiedSpritesController);
		specifiedSpritesController.addMember (rotatedSprite);
		specifiedSpritesController.addMember (colouredSquareSprite);
		specifiedSpritesController.addQTMouseListener(new DragAction (new Scaler (400)) {
			//only activates when the shift modifier is pressed
			public boolean matchModifierFilter (int mods) {
				if ((mods & 0xF) == QTConstants.kJavaShiftMask)
					return true;
				return false;
			}
		});
	//2nd BEHAVIOUR -> only alt key down when mouse pressed will allow specified sprites to be skewed when dragged
		specifiedSpritesController.addQTMouseListener(new DragAction (new Skewer (200, 200)) {
			//only activates when the alt modifier is pressed
			public boolean matchModifierFilter (int mods) {
				if ((mods & 0xF) == QTConstants.kJavaAltMask)
					return true;
				return false;
			}
		});
	//3rd BEHAVIOUR -> only alt and shift key down when mouse pressed will reset sprite's matrix
		specifiedSpritesController.addQTMouseListener(new MouseButtonAdapter () {
			//only activates when the alt AND shift modifier are pressed
			public boolean matchModifierFilter (int mods) {
				if ((mods & 0xF) == (QTConstants.kJavaAltMask | QTConstants.kJavaShiftMask))
					return true;
				return false;
			}
			
			public void mousePressed (QTMouseEvent event) {
				try {
					Transformable trans = (Transformable)event.getTarget();
					float saveX = trans.getMatrix().getTx();
					float saveY = trans.getMatrix().getTy();
					Matrix mat = new Matrix();
					mat.setTx(saveX);
					mat.setTy(saveY);
					trans.setMatrix (mat);
				} catch (QTException e) {}
			}
		});
	}
}
