/*
 * QuickTime for Java SDK Sample Code

   Usage subject to restrictions in SDK License Agreement
 * Copyright:  1996-1999 Apple Computer, Inc.

 */
package ds.actions;

import quicktime.*;
import quicktime.qd.*;
import quicktime.std.*;
import quicktime.std.image.Matrix;
import quicktime.app.actions.*;
import quicktime.app.image.Transformable;
import quicktime.app.display.QTDrawable;
import quicktime.app.anim.*;
import quicktime.app.event.*;
/**
 * This class applies a matrix rotation on the Transformable target.
 */
public class RotateMatrix extends TransformMatrix {
		
//____________________________ CLASS METHODS
	/**
	 * Create a RotateMatrix object.
	 */
	public RotateMatrix () {}


//____________________________ INSTANCE VARIABLES
	private float deltaDegree = 1;

//____________________________ INSTANCE METHODS	
	public float getDeltaDegree () { return deltaDegree; }
	
	public void setDeltaDegree (float degree) { deltaDegree = degree; }
	
	/**
	 * Applies the specified delta values to the incoming matrix. 
	 * @param theMatrix The incoming matrix is the original matrix of the target when the TransformMatrix
	 * was last initialized.
	 * @param deltaX the delta on the x-axis from the x value when the TransformMatrix
	 * was last initialized.
	 * @param deltaY the delta on the y-axis from the x value when the TransformMatrix
	 * was last initialized.
	 */
	protected void doTransformation (Matrix theMatrix, int deltaX, int deltaY) throws QTException {
		// leave this as an exercise for the reader.
		// it would be called when adding this to a DragAction
		// you could use the delta values to determine the degree of rotation
	}
	
	/**
	 * Applies the preset delta values to the incoming matrix. 
	 * @param theMatrix The incoming matrix is the current matrix of the target.
	 */
	protected void doTransformation (Matrix theMatrix) throws QTException {
		theMatrix.rotate (deltaDegree, theMatrix.getTx(), theMatrix.getTy());
	}

	/**
	 * Flips the delta values to ensure that a MatrixAction usage of this class will
	 * translate correctly when the rate at which the matrix action is invoked changes 
	 * direction.
	 */
	public void flipTransformationSettings () {
		deltaDegree = -deltaDegree;
	}
}
