/*
 * QuickTime for Java SDK Sample Code

   Usage subject to restrictions in SDK License Agreement
 * Copyright:  1996-1999 Apple Computer, Inc.

 */
package cdf;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import quicktime.app.*;
import quicktime.app.anim.*;
import quicktime.app.actions.BounceAction;
import quicktime.app.actions.NextImageAction;
import quicktime.app.actions.SimpleActionList;
import quicktime.app.display.*;
import quicktime.app.image.*;
import quicktime.std.image.*;
import quicktime.*;
import quicktime.qd.*;



public class CaptureDukeFrame {
	public QTCanvas canv;
	QTImageDrawer dukeQTDrawer;
	MultipleImagePainter dukePainter;
	int width;
	private Frame showAfter;
	private Frame addToFrame;
	private Button captureButton;
		
	public CaptureDukeFrame (Frame f, String title, int w) {

		addToFrame = f;
		width = w;
		canv = new QTCanvas(QTCanvas.kInitialSize, 0.5F, 0.5F);
		addToFrame.add("Center", canv);
					
		
		addToFrame.addNotify();
		captureButton = new Button ("QT Capture");
		captureButton.addActionListener (new ActionListener() {
			public void actionPerformed (ActionEvent ae) {
				try {
					dukePainter.setCurrentFrame(0);
					dukeQTDrawer.redraw(null);
					ImagePresenter id = dukeQTDrawer.toImagePresenter();
					ImageDescription desc = id.getDescription();
					ImageDataSequence ids = new ImageDataSequence (desc);
					ids.append (id.getImage());
					
					for (int i = 1; i < dukePainter.getNumberOfFrames(); i++) {
						dukePainter.setCurrentFrame(i);
						dukeQTDrawer.redraw(null);
						id = dukeQTDrawer.toImagePresenter();
						ids.append (id.getImage());
					}
					dukePainter.setCurrentFrame(0);

					canv.removeClient();
					dukeQTDrawer = null;	//we no longer use this
					disableButton();
					
					//create compositor
					Compositor comp = new Compositor (new QDGraphics (new QDRect(new Dimension (width, 80))), QDColor.white, 35, 1);
					Matrix matrix1 = new Matrix();
					matrix1.setTx(20);
					matrix1.setTy(0);
					TwoDSprite sprite = new TwoDSprite (ids, 1, matrix1, true, 1);
					comp.addMember(sprite);
					
					ImageSequencer seq = new ImageSequencer (ids);
					seq.setLooping (ImageSequencer.kLoopPalindrome);		
					SimpleActionList actions = new SimpleActionList();
					actions.addMember(new NextImageAction (10, 1, seq, sprite));
					actions.addMember(new BounceAction (10, 0, comp, sprite, 1.0F, 0));
					comp.addController(actions);

					comp.getTimer().setRate(1);
					canv.setClient (comp, true);
				} catch (QTException e) {
					e.printStackTrace();
				}
			}
		});
		
		addToFrame.add ("South", captureButton);
	}
	
	private void disableButton() {
		captureButton.setEnabled(false);
	}
	
	
	public void showDuke() {
		try {			
			File file = QTFactory.findAbsolutePath ("duke");
			Image[] images = getImages (file.getAbsolutePath(), 17);
			int width = images[0].getWidth(addToFrame);
			int height = images[0].getHeight(addToFrame);
			
			dukePainter = new MultipleImagePainter(images);
			dukeQTDrawer = new QTImageDrawer (dukePainter, new Dimension (width, height), Redrawable.kMultiFrame);
			dukeQTDrawer.setRedrawing(true);
			
			canv.setClient (dukeQTDrawer, true);
		} catch (Exception ex) {
			ex.printStackTrace();
			QTSession.close();
		}
	}

			
	Image[] getImages (String dir, int nimgs) throws Exception {
		MediaTracker tracker = new MediaTracker (addToFrame);
		
		File fDir = new File(dir);
		if (fDir.isDirectory() == false)
			throw new FileNotFoundException (dir);
		if (dir.charAt (dir.length() - 1) != File.separatorChar)
			dir += File.separatorChar;

		// import gif images
		Image[] images = new Image[nimgs];
		for (int i=0; i < nimgs; i++) {		
			images[i] = Toolkit.getDefaultToolkit().getImage(dir + "T" + i + ".gif");
			tracker.addImage(images[i], i);
		}
		
		// wait for them all to finish importing
		try { tracker.waitForAll(); }
		catch (InterruptedException e) { }
		
		if (tracker.isErrorAny()) throw new Exception ("Error in Media Tracker");

		// make sure images are prepared so the draw properly
		for (int i = 0; i < nimgs; i++)
			addToFrame.prepareImage (images[i], addToFrame);

		return images;
	}
}
